﻿

module("VOICE")

ASSERT(type(voice_debug_log) == "function", "Missing voice_debug_log function.")
ASSERT(type(voice_debug_log_table) == "function", "Missing voice_debug_log_table function.")

ASSERT(type(wjoin) == "function", "Missing wjoin function.")
ASSERT(type(wsplit) == "function", "Missing wsplit function.")
ASSERT(type(getFirstNChar) == "function", "Missing getFirstNChar function.")
ASSERT(type(getLastNChar) == "function", "Missing getLastNChar function.")

ASSERT(type(transform_and_format) == "function", "Missing transform_and_format function.")
ASSERT(type(transform_chain) == "function", "Missing transform_chain function.")
ASSERT(type(table_concat) == "function", "Missing table_concat function.")
ASSERT(type(transform_pattern_match) == "function", "Missing transform_pattern_match function.")

ASSERT(type(com_groups) == "table", "Missing com_groups table.")
ASSERT(type(transform.replace_sentence) == "table", "Missing transform.replace_sentence table.")
ASSERT(type(transform.road_abbrev_table) == "table", "Missing transform.road_abbrev_table table.")
ASSERT(type(transform.country_abbrev_table) == "table", "Missing transform.country_abbrev_table table.")
ASSERT(type(transform.settlement_exceptions_table) == "table", "Missing transform.settlement_exceptions_table table.")
ASSERT(type(transform.route_summary_abbrev) == "table", "Missing transform.route_summary_abbrev table.")
ASSERT(type(transform.com1) == "table", "Missing transform.com1 table.")
ASSERT(type(transform.com2) == "table", "Missing transform.com2 table.")
ASSERT(type(transform.com3) == "table", "Missing transform.com3 table.")


local function roadnumber_explode(str)
	
	local t = {}
	
	wstring.gsub(str, L"([^/]+)/?", function(s) table.insert(t, s) end)
	
	for k,_ in ipairs(t) do
		
		if wstring.find(t[k], L"^%d+$") and t[k+1] and wstring.find(t[k+1], L"^[^%d]$") then
			t[k] = t[k] .. table.remove(t, k+1)
		end
		local v = t[k]
		
		v = wstring.gsub(v, L"^0%-?(%d+)$", function(route) local out = L"O" .. route t[k] = out return out	end)
		
		if wstring.find(v, L"-") then
			
			local fi = function (route1, route2) t[k] = route1 table.insert(t, k+1, route2) end
			
			wstring.gsub(v, L"^(%d+)-(%d+)$", fi)
			
			wstring.gsub(v, L"^(%a+%d+)-(%a+%d+)$", fi)
			
			wstring.gsub(v, L"^(%a+%d+)-(%d+)$", fi)
		end
	end
	return t
end

local patterns = {
	az = {L"^[aAáÁeEéÉiIíÍoOóÓöÖőŐuUúÚüÜűŰyY]", L"^[fFlLmMnNrRsSxXyY]%s?%d+", L"^1[-%s]", L"^1%d%d%d", L"^5%d*"},
	a = {L"^[bBcCdDfFgGhHjJkKlLmMnNpPqQrRsStTvVwWxXzZ][^%s%d]", L"^[bBcCdDgGhHjJkKpPqQtTvVwWzZ]%s?%d+", L"^1%d", L"^1%d%d", L"^1%d%d%d%d", L"^[2346789]%d*"},
	
	az_szam = {L"^1%s", L"^1%d%d%d", L"^5%d*"},
	az_szoveg = {L"^[aAáÁeEéÉiIíÍoOóÓöÖőŐuUúÚüÜűŰyY]", L"^[FLNRSXY]"},
	
	a_szam = {L"^1%d", L"^1%d%d", L"^1%d%d%d%d", L"^[2346789]%d*"},
	a_szoveg = {L"^[BCDGHJKPQTVWZ]"},
}

local function pattern_match(str, patterns)
	for _,pattern in ipairs(patterns) do
		if wstring.find(str, pattern) then return true end
	end
	return false
end

local function get_article(str)
	local article
	
	_,_, str = wstring.find(str, L"^%s*(.-)%s*$")
	if pattern_match(str, patterns.az) then article = L"az "
	elseif pattern_match(str, patterns.a) then article = L"a "
	else 
		voice_debug_log(L"Nincs nevelo match: " .. str, 2)
		article = L"a "
		ASSERT(false, L"Nincs nevelo match: \"" .. str .. L"\"") 
	end
	return article
end


local roadnumber_postfixes = {
	L"úton",
	L"autópályán",
	L"útra",
	L"autópályára",
	L"utat",
	L"autópályát",
	L"út felé",
	L"autópálya felé",
}

local function inflection_roadnumber(str, inflection_typ)
	
	local routes = roadnumber_explode(str)
	
	local highway = true
	
	for k,v in ipairs(routes) do
		highway = highway and wstring.find(v, L"^[AM]%d+$")
		
		v = wstring.gsub(v, L"-", L" ")
		
		v = wstring.gsub(v, L"(%a)(%d+)", L"%1 %2")
		
		v = wstring.gsub(v, L"(%d+)(%a)", L"%1 %2")
		
		v = v .. L"-s"
		
		routes[k] = v
	end
	local postfix = L""
	if inflection_typ > 0 then 
		local com = inflection_typ - 1 
		if highway then
			highway = 2
		else
			highway = 1
		end
		postfix = L" " .. roadnumber_postfixes[2*com + highway]
	end
	local route = table_concat(routes, L", ")
	local format_str = get_article(route)..L"%s"..postfix
	return route, format_str
end

local function signpost_exit_number_name(data, idx)
	
	local article
	
	local exit_number = L""
	if data[idx].signpost.exitnumber then
		
		data[idx].signpost.exitnumber.langcode = SETTINGS.TTS.tts_langcode
		
		article = get_article(data[idx].signpost.exitnumber.text)
		
		local format_str = L"%s-s"
		
		exit_number = transform_and_format(data[idx].signpost.exitnumber, nil, format_str, true)
	end
	
	
	local exit_name = L""
	if data[idx].signpost.exitname then
		local format_str = L"%s"
		if pattern_match(data[idx].signpost.exitname.text, patterns.az_szoveg) then 
			if wstring.find(data[idx].signpost.exitname.text, L"[iI]$") then
				if not article then
					article = L"az "
				end
				format_str = L"%s"
			else
				if not article then
					article = L"az "
				end
				format_str = L"%s-i"
			end
		elseif pattern_match(data[idx].signpost.exitname.text, patterns.az_szam) then
			if not article then
				article = L"az "
			end
			format_str = L"%s-s"
		elseif pattern_match(data[idx].signpost.exitname.text, patterns.a_szoveg) then 
			if wstring.find(data[idx].signpost.exitname.text, L"[iI]$") then
				if not article then
					article = L"a "
				end
				format_str = L"%s"
			else
				if not article then
					article = L"a "
				end
				format_str = L"%s-i"
			end
		elseif pattern_match(data[idx].signpost.exitname.text, patterns.a_szam) then
			if not article then
				article = L"a "
			end
			format_str = L"%s-s"
		else
			voice_debug_log("Get article does not matched!", 2)
		end
		exit_name = transform_and_format(data[idx].signpost.exitname, nil, format_str)
	end
	
	
	if exit_number ~= L"" or exit_number ~= L"" then
		return article .. exit_number .. L", " .. exit_name .. L" kijáraton"
	end
end

local function signpost_destination(data, idx)
	if data[idx].signpost.destination then
		local format_str = L"%s felé"
		local str
		local article = get_article(data[idx].signpost.destination.text)
		str = transform.country_abbrev_table:transform(data[idx].signpost.destination.text)
		if str ~= data[idx].signpost.destination.text then
			article = L""
		end
		data[idx].signpost.destination.text = str
		format_str = article .. format_str
		return transform_and_format(data[idx].signpost.destination, nil, format_str)
	end
end

local function signpost_settlement(data, idx)
	if data[idx].signpost.settlement then
		format_str = L"%s felé"
		data[idx].signpost.settlement.text = transform.settlement_exceptions_table:transform(data[idx].signpost.settlement.text)
		return transform_and_format(data[idx].signpost.settlement, nil, format_str)
	end
end

local function signpost_roadnumber(data, idx)
	
	if data[idx].signpost.roadnumber then
		local inf_type, format_str
		
		
		if data[idx].signpost.sign_road_branch then
			inf_type = 2
		else
			inf_type = 4
		end
		data[idx].signpost.roadnumber.text, format_str = inflection_roadnumber(data[idx].signpost.roadnumber.text, inf_type)
		
		data[idx].signpost.roadnumber.langcode = SETTINGS.TTS.tts_langcode
		return transform_and_format(data[idx].signpost.roadnumber, nil, format_str, true)
	end
end

local function road_name(data, idx)
	if data[idx].road.name then 
		local format_str = L"%s"
		
		local str = transform.road_abbrev_table:transform(data[idx].road.name.text)
		
		local article = get_article(data[idx].road.name.text)
		
		local nr 
		local com = com_groups[data[idx].manouver]
		if com == 1 then
			str,nr = transform.com1:transform(str)
		elseif com == 2 then
			str,nr = transform.com2:transform(str)
		elseif com == 3 then
			str,nr = transform.com3:transform(str)
		end
		
		if nr == 0 then
			format_str = format_str .. L" felé"
		else
			
			article = get_article(str)
		end
		data[idx].road.name.text = str
		format_str = article .. format_str
		return transform_and_format(data[idx].road.name, nil, format_str)
	end
end

local function road_number(data, idx)
	if data[idx].road.number then 
		data[idx].road.number.text, format_str = inflection_roadnumber(data[idx].road.number.text, com_groups[data[idx].manouver])
		
		data[idx].road.number.langcode = SETTINGS.TTS.tts_langcode
		return transform_and_format(data[idx].road.number, nil, format_str, true)
	end
end

function format_destname(data, idx)
	local t = {}
	if data[idx].signpost then
		t = transform_chain(t, signpost_exit_number_name, 	data, idx)
		t = transform_chain(t, signpost_roadnumber, 	data, idx)
		t = transform_chain(t, signpost_destination,	data, idx)
		t = transform_chain(t, signpost_settlement, 	data, idx)
	elseif data[idx].road then
		t = transform_chain(t, road_number, 	data, idx)
		t = transform_chain(t, road_name, 	data, idx)
	end
	return table_concat(t, L", ")
end

function format_streetname(data, idx)
	local t = {}
	if data[idx].road then
		t = transform_chain(t, road_number, 	data, idx)
		t = transform_chain(t, road_name, 	data, idx)
	end
	return table_concat(t, L", ")
end

function format_sentence(str)
	for key, value in ipairs(transform.replace_sentence) do
		str = wstring.gsub(str, value[1], value[2])
	end
	return str
end


function route_summary_format_road_name(data)
	local routes = roadnumber_explode(data.text)
	
	table.sort(routes)
	
	for k,v in ipairs(routes) do
		
		v = wstring.gsub(v, L"-", L" ")
		
		v = wstring.gsub(v, L"(%a)(%d+)", L"%1 %2")
		
		v = wstring.gsub(v, L"(%d+)(%a)", L"%1 %2")
		
		v = v .. L"-s"
		
		routes[k] = v
	end
	return table_concat(routes, L", ") .. L" út"
end

function route_summary_format_street_name(data)
	
	return transform.route_summary_abbrev:transform(data.text) 
end

function route_summary_format_bridge_tunnel(data)
	return transform.road_abbrev_table:transform(data.text) 
end

function route_summary_format_order(data)
	return transform.road_abbrev_table:transform(data.text) 
end


function traffic_event_supported()
	return true
end

local tf_table = {
	{false, L"[lL]ánchíd", L"lánc híd"},
	
	{true, L"^(%a)(%d+)", L"%1 %2-s"},
	
	{true, L"^(%d+)%.?", L"%1-s"},
	
	{true, L"%((%d+)%.?", L"(%1-s"},
}

local function tf(str, article)
	local out = str
	local road = 0
	local nr
	for _,v in ipairs(tf_table) do 
		out, nr = wstring.gsub(out, v[2], v[3]) 
		
		if v[1] then road = road + nr end
	end
	
	if article or road > 0 then return get_article(str)..out
	else return out end
end

function traffic_event(DescKey, data)
	local str = translate_voice(DescKey)
	ASSERT(MODEL.regional.is_it_voice_localizable(DescKey), "Missing TrafficEvent dictionary.voice key:" .. DescKey)
	
	if data.roadnumber then 
		local road, format_str = inflection_roadnumber(data.roadnumber.text, 1)
		if data.from and not data.to then
		
			str =  str..L", "..road..L", "..tf(data.from.text)..L" közelében."
		elseif data.from and data.to then 
		
			str =  str..L", "..road..L", "..tf(data.from.text)..L" és "..tf(data.to.text)..L" között."
		else
		
			str = str..road..L"."
		end
	
	elseif data.roadname then
		local road = data.roadname.text
		
		road = get_article(road) .. transform.com1:transform(road) 
		if data.from and not data.to then
			
			local _,_, from, to = wstring.find(data.from.text, L"^([^,]+),(.+)$")
			if from and to then 
				str = str..L", "..road..L", "..tf(from, true)..L" és "..tf(to,true)..L" kereszteződésében."
			else 
				str = str..L", "..road..L", "..tf(data.from.text,true)..L" kereszteződésben."
			end
		elseif data.from and data.to then
			str = str..L", "..road..L", "..tf(data.from.text,true)..L" és "..tf(data.to.text,true)..L" között."
		else
			str = str..road..L"."
		end
	
	else
		if data.from and not data.to then
		
			str =  str..L", "..tf(data.from.text)..L" közelében."
		elseif data.from and data.to then 
		
			str =  str..L", "..tf(data.from.text)..L" és "..tf(data.to.text)..L" között."
		else
		
			str = str..L"."
		end
	end
	return str
end




local cityDistrictKeys = {
	city = "You have entered %CITYNAME.",
	district = "You have entered %DISTRICTNAME.",
	city_district = "You have entered %CITYNAME, %DISTRICTNAME.",
}

function city_or_district_changed_supported()
	if 	MODEL.regional.is_it_voice_localizable(cityDistrictKeys.city) and 
		MODEL.regional.is_it_voice_localizable(cityDistrictKeys.district) and
		MODEL.regional.is_it_voice_localizable(cityDistrictKeys.city_district) then 
		return true
	else 
		return false 
	end	
end

function city_or_district_changed( data )
	voice_debug_log( L"City or district changed, data table:", 5 )
	voice_debug_log_table( data, 5 )

	local retStr = L""		
	local localised_key = L""	
	
	
	if data.city ~= nil and data.district == nil then
		ASSERT( MODEL.regional.is_it_voice_localizable( cityDistrictKeys.city ), "Missing city-changed dictionary.voice key:" .. cityDistrictKeys.city )
		localised_key = translate_voice( m_i18n_voice( cityDistrictKeys.city ) )
		retStr = city_changed( data, localised_key )
	
	elseif data.city == nil and data.district ~= nil then
		ASSERT( MODEL.regional.is_it_voice_localizable( cityDistrictKeys.district ), "Missing district-changed dictionary.voice key:" .. cityDistrictKeys.district )
		localised_key = translate_voice( m_i18n_voice( cityDistrictKeys.district ) )
		retStr = district_changed( data, localised_key )
	
	elseif data.city ~= nil and data.district ~= nil then
		retStr = city_and_district_changed( data )
	else
		voice_debug_log( L"Untreated type is in the data table! Not city or district!", 3 )
		ASSERT( false )
	end

	voice_debug_log( L"City and/or district changed, say: " .. L"'" .. retStr .. L"'", 3 )	

	return retStr
end

function city_changed( data, localised_key )
	local cityName = L""	
	local retStr = L""	

	if data.city.phoneme then
		cityName = data.city.phoneme
		retStr = wstring.gsub( localised_key, L"CityName", cityName ) 
	elseif data.city.text then
		cityName = data.city.text
		retStr = wstring.gsub( localised_key, L"CityName", cityName )
	else
		voice_debug_log( L"Untreated data type! Not text and phoneme", 3 )
		ASSERT( false )
	end

	voice_debug_log( wstring.format( L"Raw-pattern: '%s' raw-city: '%s'" , localised_key, cityName ), 3 )

	return retStr
end

function district_changed( data, localised_key )
	local districtName = L""	
	local retStr = L""		
	local article = get_article( data.district.text )

	if data.district.phoneme then
		districtName = data.district.phoneme
		retStr = wstring.gsub( localised_key, L"DistrictName", article .. districtName .. L" kerület" )
	elseif data.district.text then
		districtName = data.district.text
		retStr = wstring.gsub( localised_key, L"DistrictName", article .. districtNumber2Ordinal( data ) .. L" kerület" )
	else
		voice_debug_log( L"Untreated data type! Not text and phoneme", 5 )
		ASSERT( false )
	end

	voice_debug_log( wstring.format( L"Raw-pattern: '%s' raw-district: '%s'" , localised_key, districtName ), 3 )

	return retStr
end

function city_and_district_changed( data )
	local cityDistrictName = L""	
	local retStr = L""		
	local localised_key = L""	

	
	if wstring.find( data.district.text, data.city.text ) then
		
		ASSERT( MODEL.regional.is_it_voice_localizable( cityDistrictKeys.district ), "Missing district-changed dictionary.voice key:" .. cityDistrictKeys.district )
		localised_key = translate_voice( m_i18n_voice( cityDistrictKeys.district ) )
		if data.district.phoneme then
			cityDistrictName = data.district.phoneme
			retStr = wstring.gsub( localised_key, L"DistrictName", cityDistrictName .. L" kerületének" )
		elseif data.district.text then
			cityDistrictName = data.district.text
			retStr = wstring.gsub( localised_key, L"DistrictName", districtNumber2Ordinal( data ) .. L" kerületének" )
		end
	
	else
		ASSERT( MODEL.regional.is_it_voice_localizable( cityDistrictKeys.city_district ), "Missing city-district-changed dictionary.voice key:" .. cityDistrictKeys.city_district )
		localised_key = translate_voice( m_i18n_voice( cityDistrictKeys.city_district ) )
		localised_key = city_changed( data, localised_key )
		retStr = district_changed( data, localised_key )
	end

	voice_debug_log( wstring.format( L"Raw-pattern: '%s' raw-city-district: '%s'" , localised_key, cityDistrictName ), 3 )

	return retStr
end

function districtNumber2Ordinal( data )
	local districtStr = data.district.text
        local districtStrLen = wstring.len( districtStr )
        local outStr = wstring.sub( districtStr, 1, 1 )
        local lastChar = wstring.sub( districtStr, 1, 1)

        for i = 2, districtStrLen do
                if ( lastChar >= L"0" and lastChar <= L"9" and wstring.sub( districtStr, i, i) == L" ") then
                        outStr = outStr .. "."
                end

                outStr = outStr .. wstring.sub( districtStr, i, i)
                lastChar = wstring.sub( districtStr, i, i)
                i = i + 1
        end

        if ( lastChar >= L"0" and lastChar <= L"9" ) then
                outStr = outStr .. L"."
        end

        return outStr
end
